@@ -1,7 +1,7 @@
 package HTML::Parser;
 
-# Copyright 1996-1999, Gisle Aas.
-# Copyright 1999, Michael A. Chase.
+# Copyright 1996-2001, Gisle Aas.
+# Copyright 1999-2000, Michael A. Chase.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
@@ -9,7 +9,7 @@ package HTML::Parser;
 use strict;
 use vars qw($VERSION @ISA);
 
-$VERSION = '3.00';  # $Date: 1999/12/14 22:31:21 $
+$VERSION = '3.1990';  # $Date: 2001/03/13 19:37:42 $
 
 require HTML::Entities;
 
@@ -17,11 +17,19 @@ require DynaLoader;
 @ISA=qw(DynaLoader);
 HTML::Parser->bootstrap($VERSION);
 
+
 sub new
 {
     my $class = shift;
     my $self = bless {}, $class;
-    _alloc_pstate($self);
+    return $self->init(@_);
+}
+
+
+sub init
+{
+    my $self = shift;
+    $self->_alloc_pstate;
 
     my %arg = @_;
     my $api_version = delete $arg{api_version} || (@_ ? 3 : 2);
@@ -105,7 +113,7 @@ sub netscape_buggy_comment  # legacy
     my $self = shift;
     if ($^W) {
 	require Carp;
-	Carp::carp("netscape_buggy_comment() is depreciated.  " .
+	Carp::carp("netscape_buggy_comment() is deprecated.  " .
 	    "Please use the strict_comment() method instead");
     }
     my $old = !$self->strict_comment;
@@ -130,13 +138,6 @@ __END__
 
 HTML::Parser - HTML parser class
 
-=head1 NOTE
-
-This is the new XS based HTML::Parser.
-It should be completely backwards compatible with
-HTML::Parser version 2.2x, but has many new features
-and is significantly faster.
-
 =head1 SYNOPSIS
 
  use HTML::Parser ();
@@ -200,7 +201,8 @@ specifications from W3C.  Where there is disagreement there is often
 an option that you can enable to get the official behaviour.
 
 The document to be parsed may be supplied in arbitrary chunks.  This
-makes on-the-fly parsing as documents are received possible.
+makes on-the-fly parsing as documents are received from the network
+possible.
 
 If event driven parsing does not feel right for your application, you
 might want to use C<HTML::TokeParser>.  It is a
@@ -219,7 +221,7 @@ The following method is used to construct a new C<HTML::Parser> object:
 This class method creates a new C<HTML::Parser> object and
 returns it.  Key/value pair arguments may be provided to assign event
 handlers or initialize parser options.  The handlers and parser
-options can also be set or modified by method calls described later.
+options can also be set or modified later by method calls described below.
 
 If a top level key is in the form "<event>_h" (e.g., "text_h"} then it
 assigns a handler to that event, otherwise it initializes a parser
@@ -270,6 +272,8 @@ to the C<HTML::Parser> object:
 
 Parse $string as the next chunk of the HTML document.  The return
 value is normally a reference to the parser object (i.e. $p).
+Handlers invoked should not attempt modify the $string in-place until
+$p->parse returns.
 
 If an invoked event handler aborts parsing by calling $p->eof, then
 $p->parse() will return a FALSE value.
@@ -350,29 +354,30 @@ since "LIST]" is not a legal attribute name.
 
 This method sets the value reported for boolean attributes inside HTML
 start tags.  By default, the name of the attribute is also used as its
-value.  This affects the values reported for C<tokens> and C<attr>.
+value.  This affects the values reported for C<tokens> and C<attr>
+argspecs.
 
 =item $p->xml_mode( [$bool] )
 
 Enabling this attribute changes the parser to allow some XML
-constructs such as empty element tags and XML processing instructions.
-It also disables forcing tag and attribute names to lower case when they
-are reported by the C<tagname> and C<attr> argspecs.
+constructs such as I<empty element tags> and I<XML processing
+instructions>.  It disables forcing tag and attribute names to lower
+case when they are reported by the C<tagname> and C<attr> argspecs,
+and suppress special treatment of elements that are parsed as CDATA
+for HTML.
 
-Empty element tags look like start tags, but end with the character
-sequence "/>".  When recognized by HTML::Parser they cause an
-artificial end event in addition to the start event.  The
-C<text> for the artificial end event will be empty
-and the C<tokenpos> array will be undefined even though
-the only element in the token array will have the correct tag name.
+I<Empty element tags> look like start tags, but end with the character
+sequence "/>".  When recognized by C<HTML::Parser> they cause an
+artificial end event in addition to the start event.  The C<text> for
+the artificial end event will be empty and the C<tokenpos> array will
+be undefined even though the only element in the token array will have
+the correct tag name.
 
-XML processing instructions are terminated by "?>" instead of a simple
-">" as is the case for HTML.
+I<XML processing instructions> are terminated by "?>" instead of a
+simple ">" as is the case for HTML.
 
 =item $p->unbroken_text( [$bool] )
 
-I<Note: This option is not supported yet!>
-
 By default, blocks of text are given to the text handler as soon as
 possible (but the parser makes sure to always break text at the
 boundary between whitespace and non-whitespace so single words and
@@ -382,7 +387,7 @@ enabled, blocks of text are always reported in one piece.  This will
 delay the text event until the following (non-text) event has been
 recognized by the parser.
 
-=item $p->marked_section( [$bool] )
+=item $p->marked_sections( [$bool] )
 
 By default, section markings like <![CDATA[...]]> are treated like
 ordinary text.  When this attribute is enabled section markings are
@@ -470,6 +475,35 @@ If there is a default handler it will be invoked.
 
 =back
 
+Filters based on tags can be set up to limit the number of events
+reported.  The main bottleneck during parsing is often the huge number
+of callbacks made.  Applying filters can improve performance
+significantly.
+
+The following methods control filters:
+
+=over
+
+=item $p->ignore_tags( TAG, ... )
+
+Any C<start> and C<end> events involving any of the tags given are
+suppressed.
+
+=item $p->report_only_tags( TAG, ... )
+
+Any C<start> and C<end> events I<not> involving any of the tags given
+are suppressed.
+
+=item $p->ignore_elements( TAG, ... )
+
+Both the C<start> and the C<end> event as well as any events that
+would be reported in between are suppressed.  The ignored elements can
+contain nested occurences of itself.  Example:
+
+   $p->ignore_elements(qw(script style));
+
+=back
+
 =head2 Argspec
 
 Argspec is a string containing a comma separated list that describes
@@ -536,6 +570,9 @@ For C<declaration> events, this is the declaration type.
 
 For C<start> and C<end> events, this is the tag name.
 
+For C<process> and non-strict C<comment> events, this is everything
+inside the tag.
+
 This passes undef if there are no tokens in the event.
 
 =item C<tagname>
@@ -580,6 +617,20 @@ This passes undef except for C<start> events.
 Unless C<xml_mode> is enabled, the attribute names are forced to lower
 case.
 
+=item C<@attr>
+
+Basically same as C<attr>, but keys and values are passed as
+individual arguments and the original sequence of the attributes is
+kept.  The parameters passed will be the same as the @attr calculated
+here:
+
+   @attr = map { $_ => $attr->{$_} } @$attrseq;
+
+assuming $attr and $attrseq here are the hash and array passed as the
+result of C<attr> and C<attrseq> argspecs.
+
+This pass no values for events besides C<start>.
+
 =item C<text>
 
 Text causes the source text (including markup element delimiters) to be
@@ -638,7 +689,7 @@ until at least one handler requests this value.
 
 =item C<'...'>
 
-A literal string of 0 to 255 chracters enclosed
+A literal string of 0 to 255 characters enclosed
 in single (') or double (") quotes is passed as entered.
 
 =item C<undef>
@@ -647,6 +698,10 @@ Pass an undefined value.  Useful as padding.
 
 =back
 
+The whole argspec string can be wrapped up in C<'@{...}'> to signal
+that resulting event array should be flatten.  This only makes a
+difference if an array reference is used as the handler target.
+
 =head2 Events
 
 Handlers for the following events can be registered:
@@ -751,7 +806,8 @@ Setup of these handlers can also be requested with the "api_version =>
 
 The C<HTML::Parser> class is subclassable.  Parser objects are plain
 hashes and C<HTML::Parser> reserves only hash keys that start with
-"_hparser".
+"_hparser".  The parser state can be set up by invoking the init()
+method which takes the same arguments as new().
 
 =head1 EXAMPLES
 
@@ -788,9 +844,10 @@ parsing as soon as the title end tag is seen:
 
 More examples are found in the "eg/" directory of the C<HTML-Parser>
 distribution; the program C<hrefsub> shows how you can edit all links
-found in a document; the program C<hstrip> shows how you can strip out
-certain tags/elements and/or attributes; and the program C<htext> show
-how to obtain the plain text, but not any script/style content.
+found in a document and C<htextsub> how to edid the text only; the
+program C<hstrip> shows how you can strip out certain tags/elements
+and/or attributes; and the program C<htext> show how to obtain the
+plain text, but not any script/style content.
 
 =head1 BUGS
 
@@ -822,7 +879,96 @@ recognized.
 
 =head1 DIAGNOSTICS
 
-[To be provided]
+The following messages may be produced by HTML::Parser.  The notation
+in this listing is the same as used in L<perldiag>:
+
+=over
+
+=item Not a reference to a hash
+
+(F) The object blessed into or subclassed from HTML::Parser is not a
+hash as required by the HTML::Parser methods.
+
+=item Bad signature in parser state object at %p
+
+(F) The _hparser_xs_state element does not refer to a valid state structure.
+Something must have changed the internal value
+stored in this hash element, or the memory has been overwritten.
+
+=item _hparser_xs_state element is not a reference
+
+(F) The _hparser_xs_state element has been destroyed.
+
+=item Can't find '_hparser_xs_state' element in HTML::Parser hash
+
+(F) The _hparser_xs_state element is missing from the parser hash.
+It was either deleted, or not created when the object was created.
+
+=item API version %s not supported by HTML::Parser %s
+
+(F) The constructor option 'api_version' with an argument greater than
+or equal to 4 is reserved for future extentions.
+
+=item Bad constructor option '%s'
+
+(F) An unknown constructor option key was passed to the new() or
+init() methods.
+
+=item Parse loop not allowed
+
+(F) A handler invoked the parse() or parse_file() method.
+This is not permitted.
+
+=item marked sections not supported
+
+(F) The $p->marked_sections() method was invoked in a HTML::Parser
+module that was compiled without support for marked sections. 
+
+=item Unknown boolean attribute (%d)
+
+(F) Something is wrong with the internal logic that set up aliases for
+boolean attributes.
+
+=item Only code or array references allowed as handler
+
+(F) The second argument for $p->handler must be either a subroutine
+reference, then name of a subroutine or method, or a reference to an
+array.
+
+=item No handler for %s events
+
+(F) The first argument to $p->handler must be a valid event name; i.e. one
+of "start", "end", "text", "process", "declaration" or "comment".
+
+=item Unrecognized identifier %s in argspec
+
+(F) The identifier is not a known argspec name.
+Use one of the names mentioned in the argspec section above.
+
+=item Literal string is longer than 255 chars in argspec
+
+(F) The current implementation limits the length of literals in
+an argspec to 255 characters.  Make the literal shorter.
+
+=item Backslash reserved for literal string in argspec
+
+(F) The backslash character "\" is not allowed in argspec literals.
+It is reserved to permit quoting inside a literal in a later version.
+
+=item Unterminated literal string in argspec
+
+(F) The terminating quote character for a literal was not found.
+
+=item Bad argspec (%s)
+
+(F) Only identifier names, literals, spaces and commas
+are allowed in argspecs.
+
+=item Missing comma separator in argspec
+
+(F) Identifiers in an argspec must be separated with ",".
+
+=back
 
 =head1 SEE ALSO
 
@@ -838,8 +984,8 @@ be found at C<http://www.sgml.u-net.com/book/sgml-8.htm>.
 
 =head1 COPYRIGHT
 
- Copyright 1996-1999 Gisle Aas. All rights reserved.
- Copyright 1999 Michael A. Chase.  All rights reserved.
+ Copyright 1996-2001 Gisle Aas. All rights reserved.
+ Copyright 1999-2000 Michael A. Chase.  All rights reserved.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
